@@ -10,11 +10,13 @@ from .cli import CommandLine, ExtendedBool
 from .str_utils import is_empty_or_whitespace
 from . import compat
 
+
 def _validate_keys(dictionary, allowed_keys):
     for key in dictionary.keys():
         if key not in allowed_keys:
             raise CrazyError(f'Unknown key: {key}')
 
+
 def dictionary_to_commandline(dictionary, prog=None):
     '''Convert a single dictionary to a cli.CommandLine object.'''
 
@@ -72,6 +74,7 @@ def dictionary_to_commandline(dictionary, prog=None):
 
     return commandline
 
+
 def _check_prog_in_dictionary(dictionary):
     if 'prog' not in dictionary:
         raise CrazyError('Missing `prog` field')
@@ -82,6 +85,7 @@ def _check_prog_in_dictionary(dictionary):
     if is_empty_or_whitespace(dictionary['prog']):
         raise CrazyError('The `prog` field must not be empty')
 
+
 def _get_commandline_by_path(root, path):
     current = root
 
@@ -97,6 +101,7 @@ def _get_commandline_by_path(root, path):
 
     return current
 
+
 def dictionaries_to_commandline(dictionaries):
     '''Convert a list of dictionaries to a cli.CommandLine object.'''
 
@@ -130,7 +135,10 @@ def dictionaries_to_commandline(dictionaries):
         progs = [c.prog for c in root.get_subcommands().subcommands]
         raise CrazyError(f"Too many main programs defined: {progs}")
 
-    return root.get_subcommands().subcommands[0]
+    cmdline = root.get_subcommands().subcommands[0]
+    cmdline.parent = None
+    return cmdline
+
 
 def option_to_dictionary(self):
     '''Convert a cli.Option object to a dictionary.'''
@@ -171,6 +179,7 @@ def option_to_dictionary(self):
 
     return r
 
+
 def positional_to_dictionary(self):
     '''Convert a cli.Positional object to a dictionary.'''
 
@@ -198,6 +207,7 @@ def positional_to_dictionary(self):
 
     return r
 
+
 def commandline_to_dictionary(commandline):
     '''Convert a cli.CommandLine object to a dictionary.'''
 
@@ -232,6 +242,7 @@ def commandline_to_dictionary(commandline):
 
     return r
 
+
 def commandline_to_dictionaries(commandline):
     '''Convert a cli.CommandLine object to dictionaries.'''
 
