@@ -392,6 +392,10 @@ if set -q files[1]
 end
 ''')
 
+_HISTORY = helpers.FishFunction('history', r'''
+builtin history | command grep -E -o -- $argv[1]
+''')
+
 # =============================================================================
 # Bonus
 # =============================================================================
@@ -409,7 +413,7 @@ _TIMEZONE_LIST = helpers.FishFunction('timezone_list', r'''
 if ! command timedatectl list-timezones 2>/dev/null
   command find /usr/share/zoneinfo -type f |\
   command sed 's|/usr/share/zoneinfo/||g'  |\
-  command egrep -v '^(posix|right)'
+  command grep -E -v '^(posix|right)'
 end''')
 
 _ALSA_LIST_CARDS = helpers.FishFunction('alsa_list_cards', r'''
@@ -436,13 +440,15 @@ for card in (command aplay -l | string match -r '^card [0-9]+: [^,]+')
   end
 end''')
 
+
 class FishHelpers(helpers.GeneralHelpers):
     '''Class holding helper functions for Fish.'''
 
     def __init__(self, function_prefix):
-        super().__init__(function_prefix)
+        super().__init__(function_prefix, helpers.FishFunction)
         self.add_function(_FISH_QUERY)
         self.add_function(_FISH_COMPLETE_FILEDIR)
+        self.add_function(_HISTORY)
         self.add_function(_NET_INTERFACES_LIST)
         self.add_function(_TIMEZONE_LIST)
         self.add_function(_ALSA_LIST_CARDS)
