@@ -1,38 +1,51 @@
 '''Contains code for preprocessing text.'''
 
+
 def preprocess(string, defines):
-    '''Simple preprocessor function.'''
+    '''Simple preprocessor function with #ifdef, #else, and #endif support.'''
 
     defines = set(defines)
     output = []
-    stack = []
+    stack = []  # stack of booleans: is this block currently active?
 
     lines = string.splitlines(keepends=True)
-    ignore = False
 
     for line in lines:
         stripped_line = line.lstrip()
+
         if stripped_line.startswith("#ifdef"):
             define = stripped_line.split()[1]
-            if define in defines:
-                stack.append(True)
-            else:
-                stack.append(False)
-                ignore = True
+            active = define in defines
+            stack.append(active)
+
+        elif stripped_line.startswith("#else"):
+            if not stack:
+                raise SyntaxError("#else without #ifdef")
+            # flip only the current (top of stack)
+            stack[-1] = not stack[-1]
+
         elif stripped_line.startswith("#endif"):
-            if stack.pop() is False:
-                ignore = False if len(stack) == 0 else not any(stack)
+            if not stack:
+                raise SyntaxError("#endif without #ifdef")
+            stack.pop()
+
         else:
-            if not ignore:
+            # output only if all enclosing blocks are active
+            if not stack or all(stack):
                 output.append(line)
 
+    if stack:
+        raise SyntaxError("Unclosed #ifdef")
+
     return ''.join(output)
 
+
 def strip_double_empty_lines(string):
     '''Collapse triple newlines into double newlines.'''
 
     return string.replace('\n\n\n', '\n\n')
 
+
 def _test():
     string = """\
 Lorem Ipsum
@@ -40,6 +53,8 @@ Lorem Ipsum
     This is debug code.
     #ifdef FOO
     This is foo code.
+    #else
+    This is foo else code.
     #endif
 #endif
 Dolor Sit Amet\
@@ -48,5 +63,6 @@ Dolor Sit Amet\
     result = preprocess(string, defines)
     print(result)
 
+
 if __name__ == '__main__':
     _test()
